//----------------------------------------------------------
#property  copyright "mladen"
#property  link      "www.forex-tsd"
//----------------------------------------------------------
#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  LimeGreen
#property  indicator_color2  DarkOrange
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_minimum 0
#property  indicator_maximum 1
//
//
//
//
//
 
extern string TimeFrame       = "current time frame";
extern int    period          = 10;
extern int    usePrice        = PRICE_MEDIAN;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double histUp[];
double histDn[];
double prices[];
double values[];
double trend[];
double ft[];

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;
   
//----------------------------------------------------------
//
//----------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,histUp); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,histDn); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,prices);
      SetIndexBuffer(3,values);
      SetIndexBuffer(4,ft);
      SetIndexBuffer(5,trend);

         //
         //
         //
         //
         //
                  
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //
         
   IndicatorShortName(timeFrameToString(timeFrame)+" Ehlers\' Fisher transform ("+period+")");
   return(0);
}


//+----------------------------------------------------------+
//|                                                          |
//+----------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = Bars-counted_bars;
           if (returnBars) { histUp[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
         
   if (calculateValue || timeFrame==Period())
   {
      for(int i=limit; i>=0; i--)
      {  
         prices[i] = iMA(NULL,0,1,0,MODE_SMA,usePrice,i);
         double MaxH = prices[ArrayMaximum(prices,period,i)];
         double MinL = prices[ArrayMinimum(prices,period,i)];
            if (MaxH!=MinL)
                  values[i] = 0.33*2*((prices[i]-MinL)/(MaxH-MinL)-0.5)+0.67*values[i+1];
            else  values[i] = 0.00;
                  values[i] = MathMin(MathMax(values[i],-0.999),0.999); 
                  ft[i]     = 0.5*MathLog((1+values[i])/(1-values[i]))+0.5*ft[i+1];

         // 
         //
         //
         //
         //

         histUp[i] = EMPTY_VALUE;
         histDn[i] = EMPTY_VALUE;
         trend[i]  = trend[i+1];
            if (ft[i]>0) trend[i] =  1;
            if (ft[i]<0) trend[i] = -1;
            if (trend[i] ==  1) histUp[i] = 1;
            if (trend[i] == -1) histDn[i] = 1;
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         histUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",period,usePrice,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
         histDn[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",period,usePrice,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
   }
   return(0);   

}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//------------------------------------------------------------------
//                                                                 
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if( alertsOn ) 
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(" up");
            if (trend[whichBar] == -1) doAlert(" down");
         }
   }
}

//
//
//
//
//

void doAlert(string doWhat )
{
   static string   previousAlert = "nothing";
   static datetime previousTime  = 0;
   string message;
   
   if(previousAlert != doWhat || previousTime != Time[0]) {
      previousAlert  = doWhat;
      previousTime   = Time[0];
      
      message =  timeFrameToString(Period())+" "+Symbol()+" Ehlers fisher transform trend changed to "+doWhat;
         if ( alertsMessage ) Alert(message);
         if ( alertsEmail ) SendMail(Symbol()+" Ehlers fisher transform "+timeFrameToString(Period()), message );
         if ( alertsSound ) PlaySound("alert2.wav");
   }
}

